/*
 * ani_encode.c
 *
 * 1.01 27-Sep-2001 Rob Voisey
 *
 * Copyright (c) SONICblue, Incorporated 2001
 *
 * Simple animation encoder
 *
 * syntax: ani_encode inputfile[00-nn].bin outputfile
 *
 */

#include <stdio.h>
#include <stdlib.h>

/*
 * Maximum number of unique frames
 */
#define MAX_FRAMES 32

/*
 * Empeg start up sequence
 */
unsigned int ani_seq[] = { 0,3,4,5,6,7,8,9,9,9,12,12,12,12,16,17,18,19,18,19,18,19,18,17,16,12,12,0xFFFF };

#if 0
/*
 * Rio start up sequence
 */
unsigned int ani_seq[] = { 0,1,2,3,4,5,6,7,7,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,21,0xFFFF };
#endif

/*
 * ani_decode : Decode a frame to screen
 *
 * char *ani : Ptr to animation data
 * char *buf : Ptr to screen data buffer
 *
 * returns : buf
 */
char *ani_decode(char *ani, char *buf)
{
	int i;

	for(i=0; i<2048; i+=2)
	{
		buf[i]   = ((ani[i] & 0xC0) >> 2) | ((ani[i] & 0x30) >> 4);
		buf[i+1] = ((ani[i] & 0x0C) << 2) | (ani[i] & 0x03);
	}

	return buf;
}

int main(int argc, char *argv[])
{
	int i, frame, max_frame=0, max_ani=0, ptr;
	int frame_list[MAX_FRAMES];
	char *fname, p1, p2;
	FILE *f_in;
	FILE *f_out;

	if(argv[1]==NULL || argv[2]==NULL)
	{
		fprintf(stderr,"Syntax: %s inputfile[00-nn].bin outputfile\n",argv[0]);
		exit(0);
	}

	if((fname = malloc(strlen(argv[1])+7))==NULL)
	{
		fprintf(stderr,"Can't allocate memory for filename!\n");
		exit(0);
	}

	if(f_out=fopen(argv[2],"wb"), f_out==NULL)
	{
		fprintf(stderr,"Unable to open destination file\n");
		exit(0);
	}

	/* Get a list of unique frames that must be processed */
	for(i=0; ani_seq[i]!=0xFFFF; i++)
	{
		for(frame=0; frame<=max_frame; frame++)
			if(ani_seq[i]==frame_list[frame])
				break;

		if(ani_seq[i]!=frame_list[frame])
			frame_list[max_frame++] = ani_seq[i];
	}

	max_ani = i+1;

	printf("%d unique frames to be encoded:\n",max_frame);
	for(i=0; i<max_frame; i++)
	  printf("%s%d",(i==0)?"":", ",frame_list[i]);
	printf("\n\n");

	/* Build the frame index */
	for(i=0; ani_seq[i]!=0xFFFF; i++)
	{
		for(frame=0; frame_list[frame]!=ani_seq[i]; frame++) ;
		ptr=(frame*1024)+(max_ani*4);
		fwrite(&ptr,4,1,f_out);
	}
	ptr=0;
	fwrite(&ptr,4,1,f_out);

	/* Add each frame */
	for(frame=0; frame<max_frame; frame++)
	{
		sprintf(fname,"%s%02d.bin",argv[1],frame_list[frame]);

		if(f_in=fopen(fname,"rb"), f_in==NULL)
		{
			fprintf(stderr,"Can't open frame file %s\n",fname);
			fclose(f_out);
			exit(0);
		}

		fseek(f_in,4,SEEK_SET);
		for(i=0; i<1024; i++)
		{
			p1 = fgetc(f_in);
			p2 = fgetc(f_in);
			fputc(((p1 & 0x30)<<2) | ((p1 & 0x03)<<4) | ((p2 & 0x30)>>2) | (p2 & 0x03),f_out);
		}

		fclose(f_in);
	}

	puts("All done.");
	fclose(f_out);
	return 0;
}
